# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long

from collections import OrderedDict
import sys
import os
import tempfile
import shutil
import zipfile
import traceback
import hashlib
from subprocess import check_output, STDOUT, CalledProcessError

from packaging.version import parse

from azure.cli.core import CommandIndex
from azure.cli.core.util import CLIError, reload_module, rmtree_with_retry
from azure.cli.core.extension import (extension_exists, build_extension_path, get_extensions, get_extension_modname,
                                      get_extension, ext_compat_with_cli, is_preview_from_extension_meta,
                                      WheelExtension, DevExtension, ExtensionNotInstalledException, WHEEL_INFO_RE)
from azure.cli.core.telemetry import set_extension_management_detail

from knack.log import get_logger

from ._homebrew_patch import HomebrewPipPatch
from ._index import get_index_extensions
from ._resolve import resolve_from_index, NoExtensionCandidatesError

logger = get_logger(__name__)

OUT_KEY_NAME = 'name'
OUT_KEY_VERSION = 'version'
OUT_KEY_TYPE = 'extensionType'
OUT_KEY_METADATA = 'metadata'
OUT_KEY_PREVIEW = 'preview'
OUT_KEY_EXPERIMENTAL = 'experimental'
OUT_KEY_PATH = 'path'

IS_WINDOWS = sys.platform.lower() in ['windows', 'win32']
LIST_FILE_PATH = os.path.join(os.sep, 'etc', 'apt', 'sources.list.d', 'azure-cli.list')
LSB_RELEASE_FILE = os.path.join(os.sep, 'etc', 'lsb-release')


def _run_pip(pip_exec_args, extension_path=None):
    cmd = [sys.executable, '-m', 'pip'] + pip_exec_args + ['--disable-pip-version-check', '--no-cache-dir']
    logger.debug('Running: %s', cmd)
    try:
        log_output = check_output(cmd, stderr=STDOUT, universal_newlines=True)
        logger.debug(log_output)
        returncode = 0
    except CalledProcessError as e:
        logger.debug(e.output)
        logger.debug(e)
        if "PermissionError: [WinError 5]" in e.output:
            logger.warning("You do not have the permission to add extensions in the target directory%s. You may need to rerun on a shell as administrator.", ': ' + os.path.split(extension_path)[0] if extension_path else '')
        returncode = e.returncode
    return returncode


def _whl_download_from_url(url_parse_result, ext_file):
    import requests
    from azure.cli.core.util import should_disable_connection_verify
    url = url_parse_result.geturl()
    r = requests.get(url, stream=True, verify=not should_disable_connection_verify())
    if r.status_code != 200:
        raise CLIError("Request to {} failed with {}".format(url, r.status_code))
    with open(ext_file, 'wb') as f:
        for chunk in r.iter_content(chunk_size=1024):
            if chunk:  # ignore keep-alive new chunks
                f.write(chunk)


def _validate_whl_extension(ext_file):
    tmp_dir = tempfile.mkdtemp()
    zip_ref = zipfile.ZipFile(ext_file, 'r')
    zip_ref.extractall(tmp_dir)
    zip_ref.close()
    azext_metadata = WheelExtension.get_azext_metadata(tmp_dir)
    rmtree_with_retry(tmp_dir)
    check_version_compatibility(azext_metadata)


def _get_extension_info_from_source(source):
    from urllib.parse import urlparse
    url_parse_result = urlparse(source)
    is_url = (url_parse_result.scheme == 'http' or url_parse_result.scheme == 'https')
    whl_filename = os.path.basename(url_parse_result.path) if is_url else os.path.basename(source)
    parsed_filename = WHEEL_INFO_RE(whl_filename)
    # Extension names can have - but .whl format changes it to _ (PEP 0427). Undo this.
    extension_name = parsed_filename.groupdict().get('name').replace('_', '-') if parsed_filename else None
    extension_version = parsed_filename.groupdict().get('ver') if parsed_filename else None
    return extension_name, extension_version


def _add_whl_ext(cli_ctx, source, ext_sha256=None, pip_extra_index_urls=None, pip_proxy=None, system=None):  # pylint: disable=too-many-statements
    from urllib.parse import urlparse
    cli_ctx.get_progress_controller().add(message='Analyzing')
    if not source.endswith('.whl'):
        raise ValueError('Unknown extension type. Only Python wheels are supported.')
    url_parse_result = urlparse(source)
    is_url = (url_parse_result.scheme == 'http' or url_parse_result.scheme == 'https')
    logger.debug('Extension source is url? %s', is_url)
    whl_filename = os.path.basename(url_parse_result.path) if is_url else os.path.basename(source)
    parsed_filename = WHEEL_INFO_RE(whl_filename)
    # Extension names can have - but .whl format changes it to _ (PEP 0427). Undo this.
    extension_name = parsed_filename.groupdict().get('name').replace('_', '-') if parsed_filename else None
    if not extension_name:
        raise CLIError('Unable to determine extension name from {}. Is the file name correct?'.format(source))
    if extension_exists(extension_name, ext_type=WheelExtension):
        raise CLIError('The extension {} already exists.'.format(extension_name))
    if extension_name == 'rdbms-connect' or extension_name == 'serviceconnector-passwordless':
        _install_deps_for_psycopg2()
    ext_file = None
    if is_url:
        # Download from URL
        tmp_dir = tempfile.mkdtemp()
        ext_file = os.path.join(tmp_dir, whl_filename)
        logger.debug('Downloading %s to %s', source, ext_file)
        import requests
        try:
            cli_ctx.get_progress_controller().add(message='Downloading')
            _whl_download_from_url(url_parse_result, ext_file)
        except (requests.exceptions.ConnectionError, requests.exceptions.HTTPError) as err:
            raise CLIError('Please ensure you have network connection. Error detail: {}'.format(str(err)))
        logger.debug('Downloaded to %s', ext_file)
    else:
        # Get file path
        ext_file = os.path.realpath(os.path.expanduser(source))
        if not os.path.isfile(ext_file):
            raise CLIError("File {} not found.".format(source))
    # Validate the extension
    logger.debug('Validating the extension %s', ext_file)
    if ext_sha256:
        valid_checksum, computed_checksum = is_valid_sha256sum(ext_file, ext_sha256)
        if valid_checksum:
            logger.debug("Checksum of %s is OK", ext_file)
        else:
            logger.debug("Invalid checksum for %s. Expected '%s', computed '%s'.",
                         ext_file, ext_sha256, computed_checksum)
            raise CLIError("The checksum of the extension does not match the expected value. "
                           "Use --debug for more information.")
    try:
        cli_ctx.get_progress_controller().add(message='Validating')
        _validate_whl_extension(ext_file)
    except AssertionError:
        logger.debug(traceback.format_exc())
        raise CLIError('The extension is invalid. Use --debug for more information.')
    except CLIError as e:
        raise e
    logger.debug('Validation successful on %s', ext_file)
    # Check for distro consistency
    check_distro_consistency()
    cli_ctx.get_progress_controller().add(message='Installing')
    # Install with pip
    extension_path = build_extension_path(extension_name, system)
    pip_args = ['install', '--target', extension_path, ext_file]

    if pip_proxy:
        pip_args = pip_args + ['--proxy', pip_proxy]
    if pip_extra_index_urls:
        for extra_index_url in pip_extra_index_urls:
            pip_args = pip_args + ['--extra-index-url', extra_index_url]

    logger.debug('Executing pip with args: %s', pip_args)
    with HomebrewPipPatch():
        pip_status_code = _run_pip(pip_args, extension_path)
    if pip_status_code > 0:
        logger.debug('Pip failed so deleting anything we might have installed at %s', extension_path)
        rmtree_with_retry(extension_path)
        raise CLIError('An error occurred. Pip failed with status code {}. '
                       'Use --debug for more information.'.format(pip_status_code))
    # Save the whl we used to install the extension in the extension dir.
    dst = os.path.join(extension_path, whl_filename)
    shutil.copyfile(ext_file, dst)
    logger.debug('Saved the whl to %s', dst)

    return extension_name


def _install_deps_for_psycopg2():  # pylint: disable=too-many-statements
    # If we are in Cloud Shell, dependencies should have already been installed.
    from azure.cli.core.util import in_cloud_console
    if in_cloud_console():
        return

    # Below system dependencies are required to install the psycopg2 dependency for Linux and macOS
    import platform
    import subprocess
    from azure.cli.core.util import get_linux_distro
    from azure.cli.core._environment import _ENV_AZ_INSTALLER
    installer = os.getenv(_ENV_AZ_INSTALLER)
    system = platform.system()
    if system == 'Darwin':
        subprocess.call(['xcode-select', '--install'], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        if installer not in ('HOMEBREW', 'HOMEBREW_CASK'):
            from shutil import which
            if which('brew') is None:
                logger.warning('You may need to install postgresql with homebrew first before you install this extension.')
                return
        exit_code = subprocess.call(['brew', 'list', 'postgresql'], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        if exit_code != 0:
            update_cmd = ['brew', 'install', 'postgresql']
            logger.warning('This extension depends on postgresql and it will be installed first.')
            logger.debug("Install dependencies with '%s'", " ".join(update_cmd))
            subprocess.call(update_cmd)
        # Fix the issue of -lssl not found during building psycopg2
        if os.environ.get('LIBRARY_PATH') is None:
            os.environ['LIBRARY_PATH'] = '/usr/local/opt/openssl/lib/'
        else:
            os.environ['LIBRARY_PATH'] = os.pathsep.join([
                os.environ.get('LIBRARY_PATH'),
                '/usr/local/opt/openssl/lib/'
            ])
    elif system == 'Linux':
        distname, _ = get_linux_distro()
        distname = distname.lower().strip()
        if installer == 'DEB' or any(x in distname for x in ['ubuntu', 'debian']):
            exit_code = subprocess.call(['dpkg', '-s', 'gcc', 'libpq-dev', 'python3-dev'], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
            if exit_code != 0:
                logger.warning('This extension depends on gcc, libpq-dev, python3-dev and they will be installed first.')
                apt_update_cmd = 'apt-get update'.split()
                apt_install_cmd = 'apt-get install -y gcc libpq-dev python3-dev'.split()
                if os.geteuid() != 0:  # pylint: disable=no-member
                    apt_update_cmd.insert(0, 'sudo')
                    apt_install_cmd.insert(0, 'sudo')
                exit_code = subprocess.call(apt_update_cmd, True)
                if exit_code == 0:
                    logger.debug("Install dependencies with '%s'", " ".join(apt_install_cmd))
                    subprocess.call(apt_install_cmd, True)
        elif installer == 'RPM' or any(x in distname for x in ['centos', 'rhel', 'red hat', 'fedora', 'opensuse', 'suse', 'sles']):
            if any(x in distname for x in ['centos', 'rhel', 'red hat', 'fedora']):
                yum_install_cmd = 'yum install -y gcc postgresql-devel python3-devel'.split()
                if os.geteuid() != 0:  # pylint: disable=no-member
                    yum_install_cmd.insert(0, 'sudo')
                logger.debug("Install dependencies with '%s'", " ".join(yum_install_cmd))
                logger.warning('This extension depends on gcc, postgresql-devel, python3-devel and they will be installed first if not exist.')
                subprocess.call(yum_install_cmd)
            elif any(x in distname for x in ['opensuse', 'suse', 'sles']):
                zypper_refresh_cmd = ['zypper', 'refresh']
                zypper_install_cmd = 'zypper install -y gcc postgresql-devel python3-devel'.split()
                logger.warning('This extension depends on gcc postgresql-devel, python3-devel and they will be installed first if not exist.')
                if os.geteuid() != 0:  # pylint: disable=no-member
                    zypper_refresh_cmd.insert(0, 'sudo')
                    zypper_install_cmd.insert(0, 'sudo')
                exit_code = subprocess.call(zypper_refresh_cmd)
                if exit_code == 0:
                    logger.debug("Install dependencies with '%s'", " ".join(zypper_install_cmd))
                    subprocess.call(zypper_install_cmd)
        elif installer == 'DOCKER' or any(x in distname for x in ['alpine linux']):
            apk_install_cmd = 'apk add --no-cache libpq-dev'.split()
            logger.debug("Install dependencies with '%s'", " ".join(apk_install_cmd))
            logger.warning('This extension depends on libpq-dev and will be installed first.')
            subprocess.call(apk_install_cmd)


def is_valid_sha256sum(a_file, expected_sum):
    sha256 = hashlib.sha256()
    with open(a_file, 'rb') as f:
        sha256.update(f.read())
    computed_hash = sha256.hexdigest()
    return expected_sum == computed_hash, computed_hash


def _augment_telemetry_with_ext_info(extension_name, ext=None):
    # The extension must be available before calling this otherwise we can't get the version from metadata
    if not extension_name:
        return
    try:
        ext = ext or get_extension(extension_name)
        ext_version = ext.version
        set_extension_management_detail(extension_name, ext_version)
    except Exception:  # nopa pylint: disable=broad-except
        # Don't error on telemetry
        pass


def check_version_compatibility(azext_metadata):
    is_compatible, cli_core_version, min_required, max_required, min_ext_required = ext_compat_with_cli(azext_metadata)
    # logger.debug("Extension compatibility result: is_compatible=%s cli_core_version=%s min_required=%s "
    #              "max_required=%s", is_compatible, cli_core_version, min_required, max_required)
    if not is_compatible:
        ext_name = azext_metadata.get('name')
        ext_version = azext_metadata.get('version')
        min_max_msgs = [
            f"The '{ext_name}' extension version {ext_version} is not compatible with your current CLI core version {cli_core_version}."
        ]
        if min_ext_required:
            min_max_msgs.append(f"This CLI core requires a min of {min_ext_required} for the '{ext_name}' extension.")
            min_max_msgs.append(f"Please run 'az extension update -n {ext_name}' to update it.")
        elif min_required and max_required:
            min_max_msgs.append(f'This extension requires a min of {min_required} and max of {max_required} CLI core.')
            min_max_msgs.append("Please run 'az upgrade' to upgrade to a compatible version.")
        elif min_required:
            min_max_msgs.append(f'This extension requires a min of {min_required} CLI core.')
            min_max_msgs.append("Please run 'az upgrade' to upgrade to a compatible version.")
        elif max_required:
            min_max_msgs.append(f'This extension requires a max of {max_required} CLI core.')
            # we do not want users to downgrade CLI core version, so we suggest updating the extension in this case
            min_max_msgs.append(f"Please run 'az extension update -n {ext_name}' to update the extension.")

        raise CLIError("\n".join(min_max_msgs))


def add_extension(cmd=None, source=None, extension_name=None, index_url=None, yes=None,  # pylint: disable=unused-argument, too-many-statements
                  pip_extra_index_urls=None, pip_proxy=None, system=None,
                  version=None, cli_ctx=None, upgrade=None, allow_preview=None):
    ext_sha256 = None
    update_to_latest = version == 'latest' and not source

    version = None if version == 'latest' else version
    cmd_cli_ctx = cli_ctx or cmd.cli_ctx
    if extension_name:
        cmd_cli_ctx.get_progress_controller().add(message='Searching')
        try:
            source, ext_sha256 = resolve_from_index(extension_name, index_url=index_url, target_version=version, cli_ctx=cmd_cli_ctx, allow_preview=allow_preview)
        except NoExtensionCandidatesError as err:
            logger.debug(err)
            err = "{}\n\nUse --debug for more information".format(err.args[0])
            raise CLIError(err)
    extension_name, ext_version = _get_extension_info_from_source(source)
    set_extension_management_detail(extension_name, ext_version)
    ext = None
    try:
        ext = get_extension(extension_name)
    except ExtensionNotInstalledException:
        pass
    if ext:
        if isinstance(ext, WheelExtension):
            logger.warning("Extension '%s' %s is already installed.", extension_name, ext.get_version())
            if not upgrade:
                return
            if ext_version == ext.get_version():
                if update_to_latest:
                    logger.warning("Latest version of '%s' is already installed.", extension_name)
                return

            logger.warning("It will be overridden with version %s.", ext_version)
            update_extension(cmd=cmd, extension_name=extension_name, index_url=index_url, pip_extra_index_urls=pip_extra_index_urls, pip_proxy=pip_proxy, cli_ctx=cli_ctx, version=ext_version, download_url=source, ext_sha256=ext_sha256)
            return
        logger.warning("Overriding development version of '%s' with production version.", extension_name)
    extension_name = _add_whl_ext(cli_ctx=cmd_cli_ctx, source=source, ext_sha256=ext_sha256,
                                  pip_extra_index_urls=pip_extra_index_urls, pip_proxy=pip_proxy, system=system)
    try:
        ext = get_extension(extension_name)
        if extension_name and ext.experimental:
            logger.warning("The installed extension '%s' is experimental and not covered by customer support. "
                           "Please use with discretion.", extension_name)
        elif extension_name and ext.preview:
            logger.warning("The installed extension '%s' is in preview.", extension_name)
        CommandIndex().invalidate()
    except ExtensionNotInstalledException:
        pass


def is_cloud_shell_system_extension(ext_path):
    from azure.cli.core.util import in_cloud_console
    if in_cloud_console():
        if ext_path.startswith('/usr/lib/python3'):
            return True
    return False


def remove_extension(extension_name):
    try:
        # Get the extension and it will raise an error if it doesn't exist
        ext = get_extension(extension_name)
        if ext and isinstance(ext, DevExtension):
            raise CLIError(
                "Extension '{name}' was installed in development mode. Remove using "
                "`azdev extension remove {name}`".format(name=extension_name))
        if is_cloud_shell_system_extension(ext.path):
            raise CLIError(f"Cannot remove system extension {extension_name} in Cloud Shell.")
        # We call this just before we remove the extension so we can get the metadata before it is gone
        _augment_telemetry_with_ext_info(extension_name, ext)
        rmtree_with_retry(ext.path)
        CommandIndex().invalidate()
    except ExtensionNotInstalledException as e:
        raise CLIError(e)


def list_extensions():
    return [{OUT_KEY_NAME: ext.name, OUT_KEY_VERSION: ext.version, OUT_KEY_TYPE: ext.ext_type,
             OUT_KEY_PREVIEW: ext.preview, OUT_KEY_EXPERIMENTAL: ext.experimental, OUT_KEY_PATH: ext.path}
            for ext in get_extensions()]


def show_extension(extension_name):
    try:
        extension = get_extension(extension_name)
        return {OUT_KEY_NAME: extension.name,
                OUT_KEY_VERSION: extension.version,
                OUT_KEY_TYPE: extension.ext_type,
                OUT_KEY_METADATA: extension.metadata,
                OUT_KEY_PATH: extension.path}
    except ExtensionNotInstalledException as e:
        raise CLIError(e)


def update_extension(cmd=None, extension_name=None, index_url=None, pip_extra_index_urls=None, pip_proxy=None, allow_preview=None, cli_ctx=None, version=None, download_url=None, ext_sha256=None):
    try:
        cmd_cli_ctx = cli_ctx or cmd.cli_ctx
        ext = get_extension(extension_name, ext_type=WheelExtension)
        if is_cloud_shell_system_extension(ext.path):
            raise CLIError(
                f"Cannot update system extension {extension_name}, please wait until Cloud Shell updates it in the next release.")
        cur_version = ext.get_version()
        try:
            if not download_url:
                download_url, ext_sha256 = resolve_from_index(extension_name, cur_version=cur_version, index_url=index_url, target_version=version, cli_ctx=cmd_cli_ctx, allow_preview=allow_preview)
            _, ext_version = _get_extension_info_from_source(download_url)
            set_extension_management_detail(extension_name, ext_version)
        except NoExtensionCandidatesError as err:
            logger.debug(err)
            msg = "{}\n\nUse --debug for more information".format(err.args[0])
            logger.warning(msg)
            return
        # Copy current version of extension to tmp directory in case we need to restore it after a failed install.
        backup_dir = os.path.join(tempfile.mkdtemp(), extension_name)
        extension_path = ext.path
        logger.debug('Backing up the current extension: %s to %s', extension_path, backup_dir)
        shutil.copytree(extension_path, backup_dir)
        # Remove current version of the extension
        rmtree_with_retry(extension_path)
        # Install newer version
        try:
            _add_whl_ext(cli_ctx=cmd_cli_ctx, source=download_url, ext_sha256=ext_sha256,
                         pip_extra_index_urls=pip_extra_index_urls, pip_proxy=pip_proxy)
            logger.debug('Deleting backup of old extension at %s', backup_dir)
            rmtree_with_retry(backup_dir)
        except Exception as err:
            logger.error('An error occurred whilst updating.')
            logger.error(err)
            logger.debug('Copying %s to %s', backup_dir, extension_path)
            shutil.copytree(backup_dir, extension_path)
            raise CLIError('Failed to update. Rolled {} back to {}.'.format(extension_name, cur_version))
        CommandIndex().invalidate()
    except ExtensionNotInstalledException as e:
        raise CLIError(e)


def list_available_extensions(index_url=None, show_details=False, cli_ctx=None):
    index_data = get_index_extensions(index_url=index_url, cli_ctx=cli_ctx)
    if show_details:
        return index_data
    installed_extensions = get_extensions(ext_type=WheelExtension)
    installed_extension_names = [e.name for e in installed_extensions]
    results = []
    for name, items in OrderedDict(sorted(index_data.items())).items():
        # exclude extensions/versions incompatible with current CLI version
        items = [item for item in items if ext_compat_with_cli(item['metadata'])[0]]
        if not items:
            continue

        latest = max(items, key=lambda c: parse(c['metadata']['version']))
        installed = False
        if name in installed_extension_names:
            installed = True
            ext_version = get_extension(name).version
            if ext_version and parse(latest['metadata']['version']) > parse(ext_version):
                installed = str(True) + ' (upgrade available)'
        results.append({
            'name': name,
            'version': latest['metadata']['version'],
            'summary': latest['metadata']['summary'],
            'preview': is_preview_from_extension_meta(latest['metadata']),
            'experimental': False,
            'installed': installed
        })
    return results


def list_versions(extension_name, index_url=None, cli_ctx=None):
    index_data = get_index_extensions(index_url=index_url, cli_ctx=cli_ctx)

    try:
        exts = index_data[extension_name]
    except Exception:
        raise CLIError('Extension {} not found.'.format(extension_name))

    try:
        installed_ext = get_extension(extension_name, ext_type=WheelExtension)
    except ExtensionNotInstalledException:
        installed_ext = None

    results = []
    latest_compatible_version = None

    for ext in sorted(exts, key=lambda c: parse(c['metadata']['version']), reverse=True):
        compatible = ext_compat_with_cli(ext['metadata'])[0]
        ext_version = ext['metadata']['version']
        if latest_compatible_version is None and compatible:
            latest_compatible_version = ext_version
        installed = ext_version == installed_ext.version if installed_ext else False
        if installed and parse(latest_compatible_version) > parse(installed_ext.version):
            installed = str(True) + ' (upgrade available)'
        version = ext['metadata']['version']
        if latest_compatible_version == ext_version:
            version = version + ' (max compatible version)'
        results.append({
            'name': extension_name,
            'version': version,
            'preview': is_preview_from_extension_meta(ext['metadata']),
            'experimental': False,
            'installed': installed,
            'compatible': compatible
        })
    results.reverse()
    return results


def reload_extension(extension_name, extension_module=None):
    return reload_module(extension_module if extension_module else get_extension_modname(ext_name=extension_name))


def add_extension_to_path(extension_name, ext_dir=None):
    ext_dir = ext_dir or get_extension(extension_name).path
    sys.path.append(ext_dir)
    # If this path update should have made a new "azure" module available,
    # extend the existing module with its path. This allows extensions to
    # include (or depend on) Azure SDK modules that are not yet part of
    # the CLI. This applies to both the "azure" and "azure.mgmt" namespaces,
    # but ensures that modules installed by the CLI take priority.
    azure_dir = os.path.join(ext_dir, "azure")
    if os.path.isdir(azure_dir):
        import azure
        azure.__path__.append(azure_dir)
        azure_mgmt_dir = os.path.join(azure_dir, "mgmt")
        if os.path.isdir(azure_mgmt_dir):
            try:
                # Should have been imported already, so this will be quick
                import azure.mgmt
            except ImportError:
                pass
            else:
                azure.mgmt.__path__.append(azure_mgmt_dir)


def get_lsb_release():
    try:
        with open(LSB_RELEASE_FILE, 'r') as lr:
            lsb = lr.readlines()
            desc = lsb[2]
            desc_split = desc.split('=')
            rel = desc_split[1]
            return rel.strip()
    except Exception:  # pylint: disable=broad-except
        return None


def check_distro_consistency():
    if IS_WINDOWS:
        return

    try:
        logger.debug('Linux distro check: Reading from: %s', LIST_FILE_PATH)

        with open(LIST_FILE_PATH, 'r') as list_file:
            package_source = list_file.read()
            stored_linux_dist_name = package_source.split(" ")[3]
            logger.debug('Linux distro check: Found in list file: %s', stored_linux_dist_name)
            current_linux_dist_name = get_lsb_release()
            logger.debug('Linux distro check: Reported by API: %s', current_linux_dist_name)

    except Exception as err:  # pylint: disable=broad-except
        current_linux_dist_name = None
        stored_linux_dist_name = None
        logger.debug('Linux distro check: An error occurred while checking '
                     'linux distribution version source list consistency.')
        logger.debug(err)

    if current_linux_dist_name != stored_linux_dist_name:
        logger.debug("Linux distro check: Mismatch distribution "
                     "name in %s file", LIST_FILE_PATH)
        logger.debug("Linux distro check: If command fails, install the appropriate package "
                     "for your distribution or change the above file accordingly.")
        logger.debug("Linux distro check: %s has '%s', current distro is '%s'",
                     LIST_FILE_PATH, stored_linux_dist_name, current_linux_dist_name)



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from typing import Any, Callable, Dict, Iterable, Optional, TypeVar

from msrest import Serializer

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.mgmt.core.exceptions import ARMErrorFormat

from .. import models as _models
from .._vendor import _convert_request, _format_url_section
T = TypeVar('T')
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False

def build_list_request(
    **kwargs: Any
) -> HttpRequest:
    api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str

    accept = "application/json"
    # Construct URL
    _url = kwargs.pop("template_url", "/providers/Microsoft.EventGrid/topicTypes")

    # Construct parameters
    _query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    _query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    # Construct headers
    _header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    _header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=_url,
        params=_query_parameters,
        headers=_header_parameters,
        **kwargs
    )


def build_get_request(
    topic_type_name: str,
    **kwargs: Any
) -> HttpRequest:
    api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str

    accept = "application/json"
    # Construct URL
    _url = kwargs.pop("template_url", "/providers/Microsoft.EventGrid/topicTypes/{topicTypeName}")
    path_format_arguments = {
        "topicTypeName": _SERIALIZER.url("topic_type_name", topic_type_name, 'str'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    _query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    _query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    # Construct headers
    _header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    _header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=_url,
        params=_query_parameters,
        headers=_header_parameters,
        **kwargs
    )


def build_list_event_types_request(
    topic_type_name: str,
    **kwargs: Any
) -> HttpRequest:
    api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str

    accept = "application/json"
    # Construct URL
    _url = kwargs.pop("template_url", "/providers/Microsoft.EventGrid/topicTypes/{topicTypeName}/eventTypes")
    path_format_arguments = {
        "topicTypeName": _SERIALIZER.url("topic_type_name", topic_type_name, 'str'),
    }

    _url = _format_url_section(_url, **path_format_arguments)

    # Construct parameters
    _query_parameters = kwargs.pop("params", {})  # type: Dict[str, Any]
    _query_parameters['api-version'] = _SERIALIZER.query("api_version", api_version, 'str')

    # Construct headers
    _header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    _header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=_url,
        params=_query_parameters,
        headers=_header_parameters,
        **kwargs
    )

class TopicTypesOperations(object):
    """TopicTypesOperations operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~azure.mgmt.eventgrid.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer):
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    @distributed_trace
    def list(
        self,
        **kwargs: Any
    ) -> Iterable["_models.TopicTypesListResult"]:
        """List topic types.

        List all registered topic types.

        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either TopicTypesListResult or the result of
         cls(response)
        :rtype: ~azure.core.paging.ItemPaged[~azure.mgmt.eventgrid.models.TopicTypesListResult]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str

        cls = kwargs.pop('cls', None)  # type: ClsType["_models.TopicTypesListResult"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        def prepare_request(next_link=None):
            if not next_link:
                
                request = build_list_request(
                    api_version=api_version,
                    template_url=self.list.metadata['url'],
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                
                request = build_list_request(
                    api_version=api_version,
                    template_url=next_link,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("TopicTypesListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return None, iter(list_of_elem)

        def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
                request,
                stream=False,
                **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response


        return ItemPaged(
            get_next, extract_data
        )
    list.metadata = {'url': "/providers/Microsoft.EventGrid/topicTypes"}  # type: ignore

    @distributed_trace
    def get(
        self,
        topic_type_name: str,
        **kwargs: Any
    ) -> "_models.TopicTypeInfo":
        """Get a topic type.

        Get information about a topic type.

        :param topic_type_name: Name of the topic type.
        :type topic_type_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: TopicTypeInfo, or the result of cls(response)
        :rtype: ~azure.mgmt.eventgrid.models.TopicTypeInfo
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.TopicTypeInfo"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str

        
        request = build_get_request(
            topic_type_name=topic_type_name,
            api_version=api_version,
            template_url=self.get.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize('TopicTypeInfo', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get.metadata = {'url': "/providers/Microsoft.EventGrid/topicTypes/{topicTypeName}"}  # type: ignore


    @distributed_trace
    def list_event_types(
        self,
        topic_type_name: str,
        **kwargs: Any
    ) -> Iterable["_models.EventTypesListResult"]:
        """List event types.

        List event types for a topic type.

        :param topic_type_name: Name of the topic type.
        :type topic_type_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either EventTypesListResult or the result of
         cls(response)
        :rtype: ~azure.core.paging.ItemPaged[~azure.mgmt.eventgrid.models.EventTypesListResult]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        api_version = kwargs.pop('api_version', "2021-10-15-preview")  # type: str

        cls = kwargs.pop('cls', None)  # type: ClsType["_models.EventTypesListResult"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        def prepare_request(next_link=None):
            if not next_link:
                
                request = build_list_event_types_request(
                    topic_type_name=topic_type_name,
                    api_version=api_version,
                    template_url=self.list_event_types.metadata['url'],
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                
                request = build_list_event_types_request(
                    topic_type_name=topic_type_name,
                    api_version=api_version,
                    template_url=next_link,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("EventTypesListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return None, iter(list_of_elem)

        def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = self._client._pipeline.run(  # pylint: disable=protected-access
                request,
                stream=False,
                **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response, error_format=ARMErrorFormat)

            return pipeline_response


        return ItemPaged(
            get_next, extract_data
        )
    list_event_types.metadata = {'url': "/providers/Microsoft.EventGrid/topicTypes/{topicTypeName}/eventTypes"}  # type: ignore



# pylint: disable=line-too-long,useless-suppression,too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AccelerationMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enable advanced network acceleration options. This allows users to configure acceleration using
    BPF host routing. This can be enabled only with Cilium dataplane. If not specified, the default
    value is None (no acceleration). The acceleration mode can be changed on a pre-existing
    cluster. See `https://aka.ms/acnsperformance <https://aka.ms/acnsperformance>`_ for a detailed
    explanation.
    """

    BPF_VETH = "BpfVeth"
    """Enable eBPF host routing with veth device mode."""
    NONE = "None"
    """Disable acceleration options."""


class AdoptionPolicy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Action if Kubernetes namespace with same name already exists."""

    NEVER = "Never"
    """If the namespace already exists in Kubernetes, attempts to create that same namespace in ARM
    will fail."""
    IF_IDENTICAL = "IfIdentical"
    """Take over the existing namespace to be managed by ARM, if there is no difference."""
    ALWAYS = "Always"
    """Always take over the existing namespace to be managed by ARM, some fields might be overwritten."""


class AdvancedNetworkPolicies(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enable advanced network policies. This allows users to configure Layer 7 network policies
    (FQDN, HTTP, Kafka). Policies themselves must be configured via the Cilium Network Policy
    resources, see `https://docs.cilium.io/en/latest/security/policy/index.html
    <https://docs.cilium.io/en/latest/security/policy/index.html>`_. This can be enabled only on
    cilium-based clusters. If not specified, the default value is FQDN if security.enabled is set
    to true.
    """

    L7 = "L7"
    """Enable Layer7 network policies (FQDN, HTTP/S, Kafka). This option is a superset of the FQDN
    option."""
    FQDN = "FQDN"
    """Enable FQDN based network policies."""
    NONE = "None"
    """Disable Layer 7 network policies (FQDN, HTTP/S, Kafka)."""


class AgentPoolMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The mode of an agent pool. A cluster must have at least one 'System' Agent Pool at all times.
    For additional information on agent pool restrictions and best practices, see:
    `https://docs.microsoft.com/azure/aks/use-system-pools
    <https://docs.microsoft.com/azure/aks/use-system-pools>`_.
    """

    SYSTEM = "System"
    """System agent pools are primarily for hosting critical system pods such as CoreDNS and
    metrics-server. System agent pools osType must be Linux. System agent pools VM SKU must have at
    least 2vCPUs and 4GB of memory."""
    USER = "User"
    """User agent pools are primarily for hosting your application pods."""
    GATEWAY = "Gateway"
    """Gateway agent pools are dedicated to providing static egress IPs to pods. For more details, see
    `https://aka.ms/aks/static-egress-gateway <https://aka.ms/aks/static-egress-gateway>`_."""


class AgentPoolSSHAccess(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """SSH access method of an agent pool."""

    LOCAL_USER = "LocalUser"
    """Can SSH onto the node as a local user using private key."""
    DISABLED = "Disabled"
    """SSH service will be turned off on the node."""


class AgentPoolType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of Agent Pool."""

    VIRTUAL_MACHINE_SCALE_SETS = "VirtualMachineScaleSets"
    """Create an Agent Pool backed by a Virtual Machine Scale Set."""
    AVAILABILITY_SET = "AvailabilitySet"
    """Use of this is strongly discouraged."""
    VIRTUAL_MACHINES = "VirtualMachines"
    """Create an Agent Pool backed by a Single Instance VM orchestration mode."""


class ArtifactSource(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The artifact source. The source where the artifacts are downloaded from."""

    CACHE = "Cache"
    """pull images from Azure Container Registry with cache."""
    DIRECT = "Direct"
    """pull images from Microsoft Artifact Registry."""


class BackendPoolType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of the managed inbound Load Balancer BackendPool."""

    NODE_IP_CONFIGURATION = "NodeIPConfiguration"
    """The type of the managed inbound Load Balancer BackendPool.
    `https://cloud-provider-azure.sigs.k8s.io/topics/loadbalancer/#configure-load-balancer-backend
    <https://cloud-provider-azure.sigs.k8s.io/topics/loadbalancer/#configure-load-balancer-backend>`_."""
    NODE_IP = "NodeIP"
    """The type of the managed inbound Load Balancer BackendPool.
    `https://cloud-provider-azure.sigs.k8s.io/topics/loadbalancer/#configure-load-balancer-backend
    <https://cloud-provider-azure.sigs.k8s.io/topics/loadbalancer/#configure-load-balancer-backend>`_."""


class Code(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Tells whether the cluster is Running or Stopped."""

    RUNNING = "Running"
    """The cluster is running."""
    STOPPED = "Stopped"
    """The cluster is stopped."""


class ConnectionStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The private link service connection status."""

    PENDING = "Pending"
    """Connection is pending approval."""
    APPROVED = "Approved"
    """Connection is approved."""
    REJECTED = "Rejected"
    """Connection is rejected."""
    DISCONNECTED = "Disconnected"
    """Connection is disconnected."""


class CreatedByType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The kind of entity that created the resource."""

    USER = "User"
    """The entity was created by a user."""
    APPLICATION = "Application"
    """The entity was created by an application."""
    MANAGED_IDENTITY = "ManagedIdentity"
    """The entity was created by a managed identity."""
    KEY = "Key"
    """The entity was created by a key."""


class DeletePolicy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Delete options of a namespace."""

    KEEP = "Keep"
    """Only delete the ARM resource, keep the Kubernetes namespace. Also delete the ManagedByARM
    label."""
    DELETE = "Delete"
    """Delete both the ARM resource and the Kubernetes namespace together."""


class Expander(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The expander to use when scaling up. If not specified, the default is 'random'. See `expanders
    <https://github.com/kubernetes/autoscaler/blob/master/cluster-autoscaler/FAQ.md#what-are-expanders>`_
    for more information.
    """

    LEAST_WASTE = "least-waste"
    """Selects the node group that will have the least idle CPU (if tied, unused memory) after
    scale-up. This is useful when you have different classes of nodes, for example, high CPU or
    high memory nodes, and only want to expand those when there are pending pods that need a lot of
    those resources."""
    MOST_PODS = "most-pods"
    """Selects the node group that would be able to schedule the most pods when scaling up. This is
    useful when you are using nodeSelector to make sure certain pods land on certain nodes. Note
    that this won't cause the autoscaler to select bigger nodes vs. smaller, as it can add multiple
    smaller nodes at once."""
    PRIORITY = "priority"
    """Selects the node group that has the highest priority assigned by the user. It's configuration
    is described in more details `here
    <https://github.com/kubernetes/autoscaler/blob/master/cluster-autoscaler/expander/priority/readme.md>`_."""
    RANDOM = "random"
    """Used when you don't have a particular need for the node groups to scale differently."""


class ExtendedLocationTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of extendedLocation."""

    EDGE_ZONE = "EdgeZone"
    """Azure Edge Zone extended location type."""


class Format(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The format of the kubeconfig credential."""

    AZURE = "azure"
    """Return azure auth-provider kubeconfig. This format is deprecated in v1.22 and will be fully
    removed in v1.26. See: `https://aka.ms/k8s/changes-1-26 <https://aka.ms/k8s/changes-1-26>`_."""
    EXEC = "exec"
    """Return exec format kubeconfig. This format requires kubelogin binary in the path."""


class GatewayAPIIstioEnabled(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Whether to enable Istio as a Gateway API implementation for managed ingress with App Routing."""

    ENABLED = "Enabled"
    """Enables managed ingress via the Gateway API using a sidecar-less Istio controlplane."""
    DISABLED = "Disabled"
    """Disables the sidecar-less istio control plane for managed ingress via the Gateway API."""


class GPUDriver(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Whether to install GPU drivers. When it's not specified, default is Install."""

    INSTALL = "Install"
    """Install driver."""
    NONE = "None"
    """Skip driver install."""


class GPUInstanceProfile(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """GPUInstanceProfile to be used to specify GPU MIG instance profile for supported GPU VM SKU."""

    MIG1_G = "MIG1g"
    """MIG 1g GPU instance profile."""
    MIG2_G = "MIG2g"
    """MIG 2g GPU instance profile."""
    MIG3_G = "MIG3g"
    """MIG 3g GPU instance profile."""
    MIG4_G = "MIG4g"
    """MIG 4g GPU instance profile."""
    MIG7_G = "MIG7g"
    """MIG 7g GPU instance profile."""


class IPFamily(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """To determine if address belongs IPv4 or IPv6 family."""

    I_PV4 = "IPv4"
    """IPv4 family."""
    I_PV6 = "IPv6"
    """IPv6 family."""


class IstioIngressGatewayMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Mode of an ingress gateway."""

    EXTERNAL = "External"
    """The ingress gateway is assigned a public IP address and is publicly accessible."""
    INTERNAL = "Internal"
    """The ingress gateway is assigned an internal IP address and cannot is accessed publicly."""


class KeyVaultNetworkAccessTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Network access of the key vault. Network access of key vault. The possible values are
    ``Public`` and ``Private``. ``Public`` means the key vault allows public access from all
    networks. ``Private`` means the key vault disables public access and enables private link. The
    default value is ``Public``.
    """

    PUBLIC = "Public"
    """Key vault allows public access from all networks."""
    PRIVATE = "Private"
    """Key vault disables public access and enables private link."""


class KubeletDiskType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Determines the placement of emptyDir volumes, container runtime data root, and Kubelet
    ephemeral storage.
    """

    OS = "OS"
    """Kubelet will use the OS disk for its data."""
    TEMPORARY = "Temporary"
    """Kubelet will use the temporary disk for its data."""


class KubernetesSupportPlan(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Different support tiers for AKS managed clusters."""

    KUBERNETES_OFFICIAL = "KubernetesOfficial"
    """Support for the version is the same as for the open source Kubernetes offering. Official
    Kubernetes open source community support versions for 1 year after release."""
    AKS_LONG_TERM_SUPPORT = "AKSLongTermSupport"
    """Support for the version extended past the KubernetesOfficial support of 1 year. AKS continues
    to patch CVEs for another 1 year, for a total of 2 years of support."""


class LicenseType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The license type to use for Windows VMs. See `Azure Hybrid User Benefits
    <https://azure.microsoft.com/pricing/hybrid-benefit/faq/>`_ for more details.
    """

    NONE = "None"
    """No additional licensing is applied."""
    WINDOWS_SERVER = "Windows_Server"
    """Enables Azure Hybrid User Benefits for Windows VMs."""


class LoadBalancerSku(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The load balancer sku for the managed cluster. The default is 'standard'. See `Azure Load
    Balancer SKUs <https://docs.microsoft.com/azure/load-balancer/skus>`_ for more information
    about the differences between load balancer SKUs.
    """

    STANDARD = "standard"
    """Use a a standard Load Balancer. This is the recommended Load Balancer SKU. For more information
    about on working with the load balancer in the managed cluster, see the `standard Load Balancer
    <https://docs.microsoft.com/azure/aks/load-balancer-standard>`_ article."""
    BASIC = "basic"
    """Use a basic Load Balancer with limited functionality."""


class LocalDNSForwardDestination(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Destination server for DNS queries to be forwarded from localDNS."""

    CLUSTER_CORE_DNS = "ClusterCoreDNS"
    """Forward DNS queries from localDNS to cluster CoreDNS."""
    VNET_DNS = "VnetDNS"
    """Forward DNS queries from localDNS to DNS server configured in the VNET. A VNET can have
    multiple DNS servers configured."""


class LocalDNSForwardPolicy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Forward policy for selecting upstream DNS server. See `forward plugin
    <https://coredns.io/plugins/forward>`_ for more information.
    """

    SEQUENTIAL = "Sequential"
    """Implements sequential upstream DNS server selection. See `forward plugin
    <https://coredns.io/plugins/forward>`_ for more information."""
    ROUND_ROBIN = "RoundRobin"
    """Implements round robin upstream DNS server selection. See `forward plugin
    <https://coredns.io/plugins/forward>`_ for more information."""
    RANDOM = "Random"
    """Implements random upstream DNS server selection. See `forward plugin
    <https://coredns.io/plugins/forward>`_ for more information."""


class LocalDNSMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Mode of enablement for localDNS."""

    PREFERRED = "Preferred"
    """If the current orchestrator version supports this feature, prefer enabling localDNS."""
    REQUIRED = "Required"
    """Enable localDNS."""
    DISABLED = "Disabled"
    """Disable localDNS."""


class LocalDNSProtocol(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enforce TCP or prefer UDP protocol for connections from localDNS to upstream DNS server."""

    PREFER_UDP = "PreferUDP"
    """Prefer UDP protocol for connections from localDNS to upstream DNS server."""
    FORCE_TCP = "ForceTCP"
    """Enforce TCP protocol for connections from localDNS to upstream DNS server."""


class LocalDNSQueryLogging(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Log level for DNS queries in localDNS."""

    ERROR = "Error"
    """Enables error logging in localDNS. See `errors plugin <https://coredns.io/plugins/errors>`_ for
    more information."""
    LOG = "Log"
    """Enables query logging in localDNS. See `log plugin <https://coredns.io/plugins/log>`_ for more
    information."""


class LocalDNSServeStale(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Policy for serving stale data. See `cache plugin <https://coredns.io/plugins/cache>`_ for more
    information.
    """

    VERIFY = "Verify"
    """Serve stale data with verification. First verify that an entry is still unavailable from the
    source before sending the expired entry to the client. See `cache plugin
    <https://coredns.io/plugins/cache>`_ for more information."""
    IMMEDIATE = "Immediate"
    """Serve stale data immediately. Send the expired entry to the client before checking to see if
    the entry is available from the source. See `cache plugin <https://coredns.io/plugins/cache>`_
    for more information."""
    DISABLE = "Disable"
    """Disable serving stale data."""


class LocalDNSState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """System-generated state of localDNS."""

    ENABLED = "Enabled"
    """localDNS is enabled."""
    DISABLED = "Disabled"
    """localDNS is disabled."""


class ManagedClusterPodIdentityProvisioningState(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """The current provisioning state of the pod identity."""

    ASSIGNED = "Assigned"
    """Pod identity is assigned."""
    CANCELED = "Canceled"
    """Pod identity assignment was canceled."""
    DELETING = "Deleting"
    """Pod identity is being deleted."""
    FAILED = "Failed"
    """Pod identity assignment failed."""
    SUCCEEDED = "Succeeded"
    """Pod identity assignment succeeded."""
    UPDATING = "Updating"
    """Pod identity is being updated."""


class ManagedClusterSKUName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The name of a managed cluster SKU."""

    BASE = "Base"
    """Base option for the AKS control plane."""
    AUTOMATIC = "Automatic"
    """Automatic clusters are optimized to run most production workloads with configuration that
    follows AKS best practices and recommendations for cluster and workload setup, scalability, and
    security. For more details about Automatic clusters see aka.ms/aks/automatic."""


class ManagedClusterSKUTier(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The tier of a managed cluster SKU. If not specified, the default is 'Free'. See `AKS Pricing
    Tier <https://learn.microsoft.com/azure/aks/free-standard-pricing-tiers>`_ for more details.
    """

    PREMIUM = "Premium"
    """Cluster has premium capabilities in addition to all of the capabilities included in 'Standard'.
    Premium enables selection of LongTermSupport (aka.ms/aks/lts) for certain Kubernetes versions."""
    STANDARD = "Standard"
    """Recommended for mission-critical and production workloads. Includes Kubernetes control plane
    autoscaling, workload-intensive testing, and up to 5,000 nodes per cluster. Guarantees 99.95%
    availability of the Kubernetes API server endpoint for clusters that use Availability Zones and
    99.9% of availability for clusters that don't use Availability Zones."""
    FREE = "Free"
    """The cluster management is free, but charged for VM, storage, and networking usage. Best for
    experimenting, learning, simple testing, or workloads with fewer than 10 nodes. Not recommended
    for production use cases."""


class ManagedGatewayType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Configuration for the managed Gateway API installation. If not specified, the default is
    'Disabled'. See `https://aka.ms/k8s-gateway-api <https://aka.ms/k8s-gateway-api>`_ for more
    details.
    """

    DISABLED = "Disabled"
    """Gateway API CRDs will not be reconciled on your cluster."""
    STANDARD = "Standard"
    """Gateway API CRDs from the standard release channel will be reconciled onto your cluster. See
    `https://aka.ms/gateway-api-versions <https://aka.ms/gateway-api-versions>`_ to see which
    bundle will be installed for your Kubernetes version."""


class NamespaceProvisioningState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The current provisioning state of the namespace."""

    UPDATING = "Updating"
    """The namespace is being updated."""
    DELETING = "Deleting"
    """The namespace is being deleted."""
    CREATING = "Creating"
    """The namespace is being created."""
    SUCCEEDED = "Succeeded"
    """The namespace provisioning succeeded."""
    FAILED = "Failed"
    """The namespace provisioning failed."""
    CANCELED = "Canceled"
    """The namespace provisioning was canceled."""


class NetworkDataplane(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Network dataplane used in the Kubernetes cluster."""

    AZURE = "azure"
    """Use Azure network dataplane."""
    CILIUM = "cilium"
    """Use Cilium network dataplane. See `Azure CNI Powered by Cilium
    <https://learn.microsoft.com/azure/aks/azure-cni-powered-by-cilium>`_ for more information."""


class NetworkMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The network mode Azure CNI is configured with. This cannot be specified if networkPlugin is
    anything other than 'azure'.
    """

    TRANSPARENT = "transparent"
    """No bridge is created. Intra-VM Pod to Pod communication is through IP routes created by Azure
    CNI. See `Transparent Mode <https://docs.microsoft.com/azure/aks/faq#transparent-mode>`_ for
    more information."""
    BRIDGE = "bridge"
    """This is no longer supported."""


class NetworkPlugin(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Network plugin used for building the Kubernetes network."""

    AZURE = "azure"
    """Use the Azure CNI network plugin. See `Azure CNI (advanced) networking
    <https://docs.microsoft.com/azure/aks/concepts-network#azure-cni-advanced-networking>`_ for
    more information."""
    KUBENET = "kubenet"
    """Use the Kubenet network plugin. See `Kubenet (basic) networking
    <https://docs.microsoft.com/azure/aks/concepts-network#kubenet-basic-networking>`_ for more
    information."""
    NONE = "none"
    """No CNI plugin is pre-installed. See `BYO CNI
    <https://docs.microsoft.com/en-us/azure/aks/use-byo-cni>`_ for more information."""


class NetworkPluginMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The mode the network plugin should use."""

    OVERLAY = "overlay"
    """Used with networkPlugin=azure, pods are given IPs from the PodCIDR address space but use Azure
    Routing Domains rather than Kubenet's method of route tables. For more information visit
    `https://aka.ms/aks/azure-cni-overlay <https://aka.ms/aks/azure-cni-overlay>`_."""


class NetworkPolicy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Network policy used for building the Kubernetes network."""

    NONE = "none"
    """Network policies will not be enforced. This is the default value when NetworkPolicy is not
    specified."""
    CALICO = "calico"
    """Use Calico network policies. See `differences between Azure and Calico policies
    <https://docs.microsoft.com/azure/aks/use-network-policies#differences-between-azure-and-calico-policies-and-their-capabilities>`_
    for more information."""
    AZURE = "azure"
    """Use Azure network policies. See `differences between Azure and Calico policies
    <https://docs.microsoft.com/azure/aks/use-network-policies#differences-between-azure-and-calico-policies-and-their-capabilities>`_
    for more information."""
    CILIUM = "cilium"
    """Use Cilium to enforce network policies. This requires networkDataplane to be 'cilium'."""


class NginxIngressControllerType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Ingress type for the default NginxIngressController custom resource."""

    ANNOTATION_CONTROLLED = "AnnotationControlled"
    """The default NginxIngressController will be created. Users can edit the default
    NginxIngressController Custom Resource to configure load balancer annotations."""
    EXTERNAL = "External"
    """The default NginxIngressController will be created and the operator will provision an external
    loadbalancer with it. Any annotation to make the default loadbalancer internal will be
    overwritten."""
    INTERNAL = "Internal"
    """The default NginxIngressController will be created and the operator will provision an internal
    loadbalancer with it. Any annotation to make the default loadbalancer external will be
    overwritten."""
    NONE = "None"
    """The default Ingress Controller will not be created. It will not be deleted by the system if it
    exists. Users should delete the default NginxIngressController Custom Resource manually if
    desired."""


class NodeOSUpgradeChannel(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Node OS Upgrade Channel. Manner in which the OS on your nodes is updated. The default is
    NodeImage.
    """

    NONE = "None"
    """No attempt to update your machines OS will be made either by OS or by rolling VHDs. This means
    you are responsible for your security updates."""
    UNMANAGED = "Unmanaged"
    """OS updates will be applied automatically through the OS built-in patching infrastructure. Newly
    scaled in machines will be unpatched initially and will be patched at some point by the OS's
    infrastructure. Behavior of this option depends on the OS in question. Ubuntu and Mariner apply
    security patches through unattended upgrade roughly once a day around 06:00 UTC. Windows does
    not apply security patches automatically and so for them this option is equivalent to None till
    further notice."""
    NODE_IMAGE = "NodeImage"
    """AKS will update the nodes with a newly patched VHD containing security fixes and bugfixes on a
    weekly cadence. With the VHD update machines will be rolling reimaged to that VHD following
    maintenance windows and surge settings. No extra VHD cost is incurred when choosing this option
    as AKS hosts the images."""
    SECURITY_PATCH = "SecurityPatch"
    """AKS downloads and updates the nodes with tested security updates. These updates honor the
    maintenance window settings and produce a new VHD that is used on new nodes. On some occasions
    it's not possible to apply the updates in place, in such cases the existing nodes will also be
    re-imaged to the newly produced VHD in order to apply the changes. This option incurs an extra
    cost of hosting the new Security Patch VHDs in your resource group for just in time
    consumption."""


class NodeProvisioningDefaultNodePools(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The set of default Karpenter NodePools (CRDs) configured for node provisioning. This field has
    no effect unless mode is 'Auto'. Warning: Changing this from Auto to None on an existing
    cluster will cause the default Karpenter NodePools to be deleted, which will drain and delete
    the nodes associated with those pools. It is strongly recommended to not do this unless there
    are idle nodes ready to take the pods evicted by that action. If not specified, the default is
    Auto. For more information see aka.ms/aks/nap#node-pools.
    """

    NONE = "None"
    """No Karpenter NodePools are provisioned automatically. Automatic scaling will not happen unless
    the user creates one or more NodePool CRD instances."""
    AUTO = "Auto"
    """A standard set of Karpenter NodePools are provisioned."""


class NodeProvisioningMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The node provisioning mode. If not specified, the default is Manual."""

    MANUAL = "Manual"
    """Nodes are provisioned manually by the user."""
    AUTO = "Auto"
    """Nodes are provisioned automatically by AKS using Karpenter (See aka.ms/aks/nap for more
    details). Fixed size Node Pools can still be created, but autoscaling Node Pools cannot be.
    (See aka.ms/aks/nap for more details)."""


class OSDiskType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The OS disk type to be used for machines in the agent pool. The default is 'Ephemeral' if the
    VM supports it and has a cache disk larger than the requested OSDiskSizeGB. Otherwise, defaults
    to 'Managed'. May not be changed after creation. For more information see `Ephemeral OS
    <https://docs.microsoft.com/azure/aks/cluster-configuration#ephemeral-os>`_.
    """

    MANAGED = "Managed"
    """Azure replicates the operating system disk for a virtual machine to Azure storage to avoid data
    loss should the VM need to be relocated to another host. Since containers aren't designed to
    have local state persisted, this behavior offers limited value while providing some drawbacks,
    including slower node provisioning and higher read/write latency."""
    EPHEMERAL = "Ephemeral"
    """Ephemeral OS disks are stored only on the host machine, just like a temporary disk. This
    provides lower read/write latency, along with faster node scaling and cluster upgrades."""


class OSSKU(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the OS SKU used by the agent pool. The default is Ubuntu if OSType is Linux. The
    default is Windows2019 when Kubernetes <= 1.24 or Windows2022 when Kubernetes >= 1.25 if OSType
    is Windows.
    """

    UBUNTU = "Ubuntu"
    """Use Ubuntu as the OS for node images."""
    AZURE_LINUX = "AzureLinux"
    """Use AzureLinux as the OS for node images. Azure Linux is a container-optimized Linux distro
    built by Microsoft, visit `https://aka.ms/azurelinux <https://aka.ms/azurelinux>`_ for more
    information."""
    AZURE_LINUX3 = "AzureLinux3"
    """Use AzureLinux3 as the OS for node images. Azure Linux is a container-optimized Linux distro
    built by Microsoft, visit `https://aka.ms/azurelinux <https://aka.ms/azurelinux>`_ for more
    information. For limitations, visit `https://aka.ms/aks/node-images
    <https://aka.ms/aks/node-images>`_. For OS migration guidance, see
    `https://aka.ms/aks/upgrade-os-version <https://aka.ms/aks/upgrade-os-version>`_."""
    CBL_MARINER = "CBLMariner"
    """Deprecated OSSKU. Microsoft recommends that new deployments choose 'AzureLinux' instead."""
    WINDOWS2019 = "Windows2019"
    """Use Windows2019 as the OS for node images. Unsupported for system node pools. Windows2019 only
    supports Windows2019 containers; it cannot run Windows2022 containers and vice versa."""
    WINDOWS2022 = "Windows2022"
    """Use Windows2022 as the OS for node images. Unsupported for system node pools. Windows2022 only
    supports Windows2022 containers; it cannot run Windows2019 containers and vice versa."""
    UBUNTU2204 = "Ubuntu2204"
    """Use Ubuntu2204 as the OS for node images, however, Ubuntu 22.04 may not be supported for all
    nodepools. For limitations and supported kubernetes versions, see
    `https://aka.ms/aks/supported-ubuntu-versions <https://aka.ms/aks/supported-ubuntu-versions>`_."""
    WINDOWS2025 = "Windows2025"
    """Use Windows2025 as the OS for node images. Unsupported for system node pools. Windows2025
    supports Windows2022 and Windows 2025 containers; it cannot run Windows2019 containers and vice
    versa."""
    UBUNTU2404 = "Ubuntu2404"
    """Use Ubuntu2404 as the OS for node images, however, Ubuntu 24.04 may not be supported for all
    nodepools. For limitations and supported kubernetes versions, see see
    `https://aka.ms/aks/supported-ubuntu-versions <https://aka.ms/aks/supported-ubuntu-versions>`_."""
    AZURE_CONTAINER_LINUX = "AzureContainerLinux"
    """Use Azure Container Linux as the OS for node images. Azure Container Linux is a
    container-optimized, security-focused Linux OS built on Azure Linux, with an immutable
    filesystem. ACL is derived from the Flatcar Container Linux project, building on Flatcar's
    proven container-first, immutable design, while adding Azure Linux packages, servicing, and
    deep integration with the Azure and AKS lifecycle. For more information, see
    `https://aka.ms/azurecontainerlinux <https://aka.ms/azurecontainerlinux>`_."""


class OSType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The operating system type. The default is Linux."""

    LINUX = "Linux"
    """Use Linux."""
    WINDOWS = "Windows"
    """Use Windows."""


class OutboundType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The outbound (egress) routing method. This can only be set at cluster creation time and cannot
    be changed later. For more information see `egress outbound type
    <https://docs.microsoft.com/azure/aks/egress-outboundtype>`_.
    """

    LOAD_BALANCER = "loadBalancer"
    """The load balancer is used for egress through an AKS assigned public IP. This supports
    Kubernetes services of type 'loadBalancer'. For more information see `outbound type
    loadbalancer
    <https://docs.microsoft.com/azure/aks/egress-outboundtype#outbound-type-of-loadbalancer>`_."""
    USER_DEFINED_ROUTING = "userDefinedRouting"
    """Egress paths must be defined by the user. This is an advanced scenario and requires proper
    network configuration. For more information see `outbound type userDefinedRouting
    <https://docs.microsoft.com/azure/aks/egress-outboundtype#outbound-type-of-userdefinedrouting>`_."""
    MANAGED_NAT_GATEWAY = "managedNATGateway"
    """The AKS-managed NAT gateway is used for egress."""
    USER_ASSIGNED_NAT_GATEWAY = "userAssignedNATGateway"
    """The user-assigned NAT gateway associated to the cluster subnet is used for egress. This is an
    advanced scenario and requires proper network configuration."""
    NONE = "none"
    """The AKS cluster is not set with any outbound-type. All AKS nodes follows Azure VM default
    outbound behavior. Please refer to
    `https://azure.microsoft.com/en-us/updates/default-outbound-access-for-vms-in-azure-will-be-retired-transition-to-a-new-method-of-internet-access/
    <https://azure.microsoft.com/en-us/updates/default-outbound-access-for-vms-in-azure-will-be-retired-transition-to-a-new-method-of-internet-access/>`_."""


class PodIPAllocationMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Pod IP Allocation Mode. The IP allocation mode for pods in the agent pool. Must be used with
    podSubnetId. The default is 'DynamicIndividual'.
    """

    DYNAMIC_INDIVIDUAL = "DynamicIndividual"
    """Each node gets allocated with a non-contiguous list of IP addresses assignable to pods. This is
    better for maximizing a small to medium subnet of size /16 or smaller. The Azure CNI cluster
    with dynamic IP allocation defaults to this mode if the customer does not explicitly specify a
    podIPAllocationMode."""
    STATIC_BLOCK = "StaticBlock"
    """Each node is statically allocated CIDR block(s) of size /28 = 16 IPs per block to satisfy the
    maxPods per node. Number of CIDR blocks >= (maxPods / 16). The block, rather than a single IP,
    counts against the Azure Vnet Private IP limit of 65K. Therefore block mode is suitable for
    running larger workloads with more than the current limit of 65K pods in a cluster. This mode
    is better suited to scale with larger subnets of /15 or bigger."""


class PolicyRule(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enum representing different network policy rules."""

    DENY_ALL = "DenyAll"
    """Deny all network traffic."""
    ALLOW_ALL = "AllowAll"
    """Allow all network traffic."""
    ALLOW_SAME_NAMESPACE = "AllowSameNamespace"
    """Allow traffic within the same namespace."""


class PrivateEndpointConnectionProvisioningState(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """The current provisioning state."""

    CANCELED = "Canceled"
    """Private endpoint connection provisioning was canceled."""
    CREATING = "Creating"
    """Private endpoint connection is being created."""
    DELETING = "Deleting"
    """Private endpoint connection is being deleted."""
    FAILED = "Failed"
    """Private endpoint connection provisioning failed."""
    SUCCEEDED = "Succeeded"
    """Private endpoint connection provisioning succeeded."""


class Protocol(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The network protocol of the port."""

    TCP = "TCP"
    """TCP protocol."""
    UDP = "UDP"
    """UDP protocol."""


class ProxyRedirectionMechanism(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Mode of traffic redirection."""

    INIT_CONTAINERS = "InitContainers"
    """Istio will inject an init container into each pod to redirect traffic (requires NET_ADMIN and
    NET_RAW)."""
    CNI_CHAINING = "CNIChaining"
    """Istio will install a chained CNI plugin to redirect traffic (recommended)."""


class PublicNetworkAccess(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """PublicNetworkAccess of the managedCluster. Allow or deny public network access for AKS."""

    ENABLED = "Enabled"
    """Inbound/Outbound to the managedCluster is allowed."""
    DISABLED = "Disabled"
    """Inbound traffic to managedCluster is disabled, traffic from managedCluster is allowed."""


class ResourceIdentityType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of identity used for the managed cluster. For more information see `use managed
    identities in AKS <https://docs.microsoft.com/azure/aks/use-managed-identity>`_.
    """

    SYSTEM_ASSIGNED = "SystemAssigned"
    """Use an implicitly created system assigned managed identity to manage cluster resources. Master
    components in the control plane such as kube-controller-manager will use the system assigned
    managed identity to manipulate Azure resources."""
    USER_ASSIGNED = "UserAssigned"
    """Use a user-specified identity to manage cluster resources. Master components in the control
    plane such as kube-controller-manager will use the specified user assigned managed identity to
    manipulate Azure resources."""
    NONE = "None"
    """Do not use a managed identity for the Managed Cluster, service principal will be used instead."""


class RestrictionLevel(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The restriction level applied to the cluster's node resource group. If not specified, the
    default is 'Unrestricted'.
    """

    UNRESTRICTED = "Unrestricted"
    """All RBAC permissions are allowed on the managed node resource group."""
    READ_ONLY = "ReadOnly"
    """Only \\*\\/read RBAC permissions allowed on the managed node resource group."""


class ScaleDownMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Describes how VMs are added to or removed from Agent Pools. See `billing states
    <https://docs.microsoft.com/azure/virtual-machines/states-billing>`_.
    """

    DELETE = "Delete"
    """Create new instances during scale up and remove instances during scale down."""
    DEALLOCATE = "Deallocate"
    """Attempt to start deallocated instances (if they exist) during scale up and deallocate instances
    during scale down."""


class ScaleSetEvictionPolicy(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The Virtual Machine Scale Set eviction policy. The eviction policy specifies what to do with
    the VM when it is evicted. The default is Delete. For more information about eviction see `spot
    VMs <https://docs.microsoft.com/azure/virtual-machines/spot-vms>`_.
    """

    DELETE = "Delete"
    """Nodes in the underlying Scale Set of the node pool are deleted when they're evicted."""
    DEALLOCATE = "Deallocate"
    """Nodes in the underlying Scale Set of the node pool are set to the stopped-deallocated state
    upon eviction. Nodes in the stopped-deallocated state count against your compute quota and can
    cause issues with cluster scaling or upgrading."""


class ScaleSetPriority(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The Virtual Machine Scale Set priority."""

    SPOT = "Spot"
    """Spot priority VMs will be used. There is no SLA for spot nodes. See `spot on AKS
    <https://docs.microsoft.com/azure/aks/spot-node-pool>`_ for more information."""
    REGULAR = "Regular"
    """Regular VMs will be used."""


class ServiceMeshMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Mode of the service mesh."""

    ISTIO = "Istio"
    """Istio deployed as an AKS addon."""
    DISABLED = "Disabled"
    """Mesh is disabled."""


class SnapshotType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of a snapshot. The default is NodePool."""

    NODE_POOL = "NodePool"
    """The snapshot is a snapshot of a node pool."""


class TransitEncryptionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Configures pod-to-pod encryption. This can be enabled only on Cilium-based clusters. If not
    specified, the default value is None.
    """

    WIRE_GUARD = "WireGuard"
    """Enable WireGuard encryption. Refer to
    `https://docs.cilium.io/en/latest/security/network/encryption-wireguard/
    <https://docs.cilium.io/en/latest/security/network/encryption-wireguard/>`_ on use cases and
    implementation details."""
    NONE = "None"
    """Disable Transit encryption."""


class TrustedAccessRoleBindingProvisioningState(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """The current provisioning state of trusted access role binding."""

    CANCELED = "Canceled"
    """Trusted access role binding provisioning was canceled."""
    DELETING = "Deleting"
    """Trusted access role binding is being deleted."""
    FAILED = "Failed"
    """Trusted access role binding provisioning failed."""
    SUCCEEDED = "Succeeded"
    """Trusted access role binding provisioning succeeded."""
    UPDATING = "Updating"
    """Trusted access role binding is being updated."""


class Type(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The week index. Specifies on which week of the month the dayOfWeek applies."""

    FIRST = "First"
    """First week of the month."""
    SECOND = "Second"
    """Second week of the month."""
    THIRD = "Third"
    """Third week of the month."""
    FOURTH = "Fourth"
    """Fourth week of the month."""
    LAST = "Last"
    """Last week of the month."""


class UndrainableNodeBehavior(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Defines the behavior for undrainable nodes during upgrade. The most common cause of undrainable
    nodes is Pod Disruption Budgets (PDBs), but other issues, such as pod termination grace period
    is exceeding the remaining per-node drain timeout or pod is still being in a running state, can
    also cause undrainable nodes.
    """

    CORDON = "Cordon"
    """AKS will cordon the blocked nodes and replace them with surge nodes during upgrade. The blocked
    nodes will be cordoned and replaced by surge nodes. The blocked nodes will have label
    'kubernetes.azure.com/upgrade-status:Quarantined'. A surge node will be retained for each
    blocked node. A best-effort attempt will be made to delete all other surge nodes. If there are
    enough surge nodes to replace blocked nodes, then the upgrade operation and the managed cluster
    will be in failed state. Otherwise, the upgrade operation and the managed cluster will be in
    canceled state."""
    SCHEDULE = "Schedule"
    """AKS will mark the blocked nodes schedulable, but the blocked nodes are not upgraded. A
    best-effort attempt will be made to delete all surge nodes. The upgrade operation and the
    managed cluster will be in failed state if there are any blocked nodes."""


class UpgradeChannel(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The upgrade channel for auto upgrade. The default is 'none'. For more information see `setting
    the AKS cluster auto-upgrade channel
    <https://docs.microsoft.com/azure/aks/upgrade-cluster#set-auto-upgrade-channel>`_.
    """

    RAPID = "rapid"
    """Automatically upgrade the cluster to the latest supported patch release on the latest supported
    minor version. In cases where the cluster is at a version of Kubernetes that is at an N-2 minor
    version where N is the latest supported minor version, the cluster first upgrades to the latest
    supported patch version on N-1 minor version. For example, if a cluster is running version
    1.17.7 and versions 1.17.9, 1.18.4, 1.18.6, and 1.19.1 are available, your cluster first is
    upgraded to 1.18.6, then is upgraded to 1.19.1."""
    STABLE = "stable"
    """Automatically upgrade the cluster to the latest supported patch release on minor version N-1,
    where N is the latest supported minor version. For example, if a cluster is running version
    1.17.7 and versions 1.17.9, 1.18.4, 1.18.6, and 1.19.1 are available, your cluster is upgraded
    to 1.18.6."""
    PATCH = "patch"
    """Automatically upgrade the cluster to the latest supported patch version when it becomes
    available while keeping the minor version the same. For example, if a cluster is running
    version 1.17.7 and versions 1.17.9, 1.18.4, 1.18.6, and 1.19.1 are available, your cluster is
    upgraded to 1.17.9."""
    NODE_IMAGE = "node-image"
    """Automatically upgrade the node image to the latest version available. Consider using
    nodeOSUpgradeChannel instead as that allows you to configure node OS patching separate from
    Kubernetes version patching."""
    NONE = "none"
    """Disables auto-upgrades and keeps the cluster at its current version of Kubernetes."""


class WeekDay(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The weekday enum."""

    SUNDAY = "Sunday"
    """Represents Sunday."""
    MONDAY = "Monday"
    """Represents Monday."""
    TUESDAY = "Tuesday"
    """Represents Tuesday."""
    WEDNESDAY = "Wednesday"
    """Represents Wednesday."""
    THURSDAY = "Thursday"
    """Represents Thursday."""
    FRIDAY = "Friday"
    """Represents Friday."""
    SATURDAY = "Saturday"
    """Represents Saturday."""


class WorkloadRuntime(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Determines the type of workload a node can run."""

    OCI_CONTAINER = "OCIContainer"
    """Nodes will use Kubelet to run standard OCI container workloads."""
    WASM_WASI = "WasmWasi"
    """Nodes will use Krustlet to run WASM workloads using the WASI provider (Preview)."""
    KATA_VM_ISOLATION = "KataVmIsolation"
    """Nodes can use (Kata + Cloud Hypervisor + Hyper-V) to enable Nested VM-based pods. Due to the
    use Hyper-V, AKS node OS itself is a nested VM (the root OS) of Hyper-V. Thus it can only be
    used with VM series that support Nested Virtualization such as Dv3 series."""



#-------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#--------------------------------------------------------------------------

def get_cli_active_cloud():
    """Return a CLI active cloud.

    .. versionadded:: 1.1.6

    :return: A CLI Cloud
    :rtype: azure.cli.core.cloud.Cloud
    :raises: ImportError if azure-cli-core package is not available
    """

    try:
        from azure.cli.core.cloud import get_active_cloud
    except ImportError:
        raise ImportError("You need to install 'azure-cli-core' to load CLI active Cloud")
    return get_active_cloud()

